;****************************************************************
;*               PgmrCalc - Programmers Calculator              *
;*                                                              *
;*                                                              *
;*  Written by:  Gary T. Desrosiers                             *
;*               desrosi@pcnet.com                              *
;*               http://www.pcnet.com/~desrosi                  *
;*                                                              *
;*  Date:        April 8th, 1997                                *
;*                                                              *
;*  Description: Programmers calculator with base conversion    *
;*               functions.                                     *
;*                                                              *
;*                                                              *
;*  Modified by: Art Dahm                                       *
;*               palm@dahm.com                                  *
;*               http://palm.dahm.com                           *
;*                                                              *
;*  Date:        May 16, 1998                                   *
;*                                                              *
;*  Description: - Added memory functions (M+, MC, MR).         *
;*               - Modified keyboard layout.                    *
;*               - Display is reset when user enters numbers    *
;*                 after unary operations and base conversions. *
;*               - Most Boolean operations now work in Decimal  *
;*                 mode (exceptions: Left Rotate & Right        *
;*                 Rotate).                                     *
;*                                                              *
;*  Date:        August 3, 1998                                 *
;*                                                              *
;*  Description: - Improved memory button behavior.             *
;*                                                              *
;****************************************************************

        Appl    "PgmrCalc", 'PgMc'
        include "Pilot.inc"
        include "Startup.inc"
	include "pgmrcalc.inc"

struct FloatType
man.l                           ;unsigned 32 bit
exp.w                           ;signed 16 bit
sign.b                          ;+/- 1	
endstruct

        data
 	align 4			;Floating variables
global Mem.FloatType		;Memory
 	align 4
global Op1.FloatType		;Operand 1
 	align 4
global Op2.FloatType		;Operand 2
 	align 4
global fpnTemp.FloatType	;Temporary float
	align 4
Zero   dc.l 0			;Floating 0.0 constant
       dc.w 0
       dc.b 1,0
	align 4
global bOp1.l			;binary counterparts
global bOp2.l			; to the floating operands
global CurrentBase.w		;1 - Decimal, 0 - Hex
global NumOps.b			;Number of operands
global DecimalFlag.b		;Decimal Point Present Yet?
global LastWasOp.b		;Last input num (0) or op(1)
global LastOp.b			;Last operation + - x /
global LastWasUnary.b
global LastWasMem.b
memchr	dc.b 'M',0
memclrchr	dc.b ' ',0
opTable	dc.b ' +-*/'
opTable2 dc.b 'AND OR  XOR MOD '
alpha	dc.b	'ABCDEF'
instr   ds.b	2
global ReadOut.40		;What's displayed
global TempBuffer.40		;temporary buffer space

        code

proc PilotMain(cmd.w, cmdPBP.l, launchFlags.w)
local   err.w
local   evt.EventType
beginproc
        tst.w   cmd(a6)      				        ;sysAppLaunchCmdNormalLaunch is 0
        bne     Return                 				;not a normal launch, bag out
        systrap FplInit()					;initialize floating point environment
	call	Initialization()				;Initialize calculator
	clr.l	FloatType.man+Mem(a5)
	clr.w	FloatType.exp+Mem(a5)
	move.b	#1,FloatType.sign+Mem(a5)
	move.w	#10,CurrentBase(a5)				;mode is initially decimal
        systrap FrmGotoForm(#kidfMain.w)
EvtLoop
	systrap EvtGetEvent(&evt(a6), #evtWaitForever.w)
        systrap SysHandleEvent(&evt(a6))
        tst.b   d0						;handled?
        bne.s   EvtDone						;yep
        systrap MenuHandleEvent(&0, &evt(a6),&err(a6))
        tst.b   d0						;handled?
        bne.s   EvtDone						;yep
        call    MainFormHandleEvent(&evt(a6))
        tst.b   d0                    				;handled?
        bne.s   EvtDone           				;yep
        systrap FrmGetActiveForm()
        systrap FrmHandleEvent(a0.l, &evt(a6))
EvtDone cmpi.w  #appStopEvent,evt+EventType.eType(a6) 		;time to stop?
        bne.s   EvtLoop               				;nope, loop until it is
        systrap FplFree()					;clean up floating point env.
Return  moveq   #0,d0
endproc

 
proc MainFormHandleEvent(pevt.l)
local rect.RectangleType
beginproc
        movem.l a3-a4/d2,-(a7)             			;save regs
        movea.l pevt(a6),a0					;a0 = pevt
        move.w  EventType.eType(a0),d0
        cmp.w	#frmLoadEvent,d0 				;frmLoadEvent?
        bne.s   MFH0
        systrap FrmInitForm(EventType.data+frmLoad.formID(a0).w)
        systrap FrmSetActiveForm(a0.l)
	bra	MFHEat

MFH0	cmp.w   #frmOpenEvent,d0
        bne.s	NotOpen
        systrap FrmGetActiveForm()
	move.l	a0,a3
	systrap	FrmSetControlGroupSelection(a3.l,#1.b,#10.w) 	;Turn on decimal pushbutton
        systrap FrmDrawForm(a3.l)
	move.w	#8,rect.topLeft.x(a6)
	move.w	#5,rect.topLeft.y(a6)
	move.w	#140,rect.extent.x(a6)
	move.w	#25,rect.extent.y(a6)
	systrap	WinDrawRectangleFrame(#5.w,&rect(a6))		;Draw the frame around the display
	call	UpdateDisplay(&Zero(a5))			;update display with inital zero
        bra     MFHEat

NotOpen cmp.w   #menuEvent,d0
        bne     MFH1
        cmp.w   #kidmAbout,EventType.data+menu.itemID(a0)
        bne     OpKey						;if not about then process as operation
        systrap FrmHelp(#kidrAboutAlert.w)			;display about dialog
        bra     MFHEat


MFH1	cmp.w	#ctlEnterEvent,d0
	bne	MFH2
	cmp.w	#kidcbackspace,EventType.data+ctlSelect.controlID(a0)	;BS key right of display?
	bne	MFHNotHandled					;no other EnterEvents to do
Bkspc	tst.b	ReadOut(a5)					;is the display empty?
	beq	beep
	cmp.w	#10,CurrentBase(a5)				;Decimal?
	beq.s	BSDec						;Dec uses floating point, else binary
	call	InputConversion(&ReadOut(a5),CurrentBase(a5).w) ;Convert display to binary in d0
	move.w	CurrentBase(a5),d1				;divide by
	ext.l	d1						; current base
	call	LongDivide(d0.l,d1.l)				;  to shift right
	call	UpdateDisplay(d0.l)				;And update display with new value
	bra	BSClick						;Make it audible
BSDec
	systrap	StrLen(&ReadOut(a5))				;get length of display
	subq.l	#1,d0						;Basckspace by decrementing length
	cmp.b	#'.',ReadOut(a5,d0)				;are we at a dot?
	bne.s	BSNDot						;no, continue
	clr.b	DecimalFlag(a5)					;otherwise, clear decimal pt flag
BSNDot	clr.b	ReadOut(a5,d0)					;set end of string
        systrap FplAToF(&fpnTemp(a5),&ReadOut(a5))		;convert display to fpnTemp
	call	UpdateDisplay(&fpnTemp(a5))			;Update display with new value
BSClick
	clr.b	LastWasOp(a5)					;set last input to num
	clr.b	LastWasUnary(a5)				;set last input to num
	clr.b	LastWasMem(a5)
	systrap SndPlaySystemSound(#sndClick.b)			;make a clicking noise
	bra	MFHEat						;end of event

MFH2	cmpi.w	#keyDownEvent,d0				;user wrote in the giraffe area?
	bne	MFH3						;no, then check next event type
	move.w	EventType.data+keyDown.chr(a0),d0		;get key
	cmpi.w	#08,d0						;see if backspace?
	beq	Bkspc						;yes, it's backspace
	cmpi.w	#'.',d0						;a dot?
	beq	Dot						;yes, then process it
	cmpi.w	#'0',d0						;0-9?
	blt	beep
	cmpi.w	#'9',d0		
	ble	SimKey
	cmpi.w	#'A',d0						;A-F?
	blt	beep
	cmpi.w	#'F',d0
	ble	SimKey
	cmpi.w	#'a',d0
	blt	beep
	cmpi.w	#'f',d0
	bgt	beep
	and.w	#$00df,d0					;Convert lowercase a-f to A-F
	bra	SimKey						;simulate the button

MFH3	cmp.w   #ctlSelectEvent,d0				;select events for button presses?
        bne     MFHNotHandled					;no other eventtypes to process.
OpKey	cmp.w   #kidchex,EventType.data+ctlSelect.controlID(a0) ;check if Dec, Hex, Bin, or Oct
	bgt.s 	NotBase
	move.b	#1,LastWasUnary(a5)
	;resources are 2, 8, 10, or 16
	call	ConvertBaseToBase(CurrentBase(a5).w,EventType.data+ctlSelect.controlID(a0).w)
        movea.l pevt(a6),a0					;a0 = pevt
	move.w	EventType.data+ctlSelect.controlID(a0),CurrentBase(a5) ;set currentbase
	bra	NumDisp						;go display result

NotBase	cmp.w   #kidbdot,EventType.data+ctlSelect.controlID(a0) ;Dot pressed?
	bne.s 	NoDot						;no, try next key
Dot	cmp.w	#10,CurrentBase(a5)				;are we in decimal mode?
	bne.s	beep						;dot is invalid key when not in dec mode
	tst.b	LastWasMem(a5)
	bne.s	DotClear
	tst.b	LastWasOp(a5)					;check if last thing was operation?
	beq.s	DotCont						;no, then continue
DotClear
	clr.b	ReadOut(a5)					;else clear display first
	bra.s	DotSet						;and go set decimalflag
DotCont	tst.b	DecimalFlag(a5)					;already dot?
	beq.s	DotSet						;no, then set flag
beep								;user pressed dot more than once
	systrap SndPlaySystemSound(#sndError.b)			; in a single numeric input
	bra	MFHEat

DotSet	move.b	#1,DecimalFlag(a5)				;decimal encountered flag
	move.w	EventType.data+ctlSelect.controlID(a0),d0	;move buttonpress event to d0
	bra.s	NumInp						;continue, this is numeric input

NoDot  	move.w	EventType.data+ctlSelect.controlID(a0),d0	;get key
	cmp.w   #kidb0,d0					;Is it 0-9, A-F?
	blt	NoDigit
        cmp.w   #kidb9,d0
	ble.s 	SimKey
	cmp.w	#kidbA,d0
	blt	NoDigit
	cmp.w	#kidbF,d0
	bgt	NoDigit
		
SimKey	move.w	d0,d1						;move to d1
	sub.w	#$30,d1						;Resources are Ascii values
	cmp.w	#$9,d1						;if not num
	ble.s	DgtNum						;  then A-F
	sub.w	#7,d1						;     sub 7 to make $00-$0F
DgtNum	cmp.w	CurrentBase(a5),d1				;compare key to base
	bge.s	beep						;key must be less than selected base
	tst.b	LastWasMem(a5)
	bne.s	NumClr
	tst.b	LastWasOp(a5)					;was last input operation?
	bne.s	NumClr						;no, then continue
	tst.b	LastWasUnary(a5)
	beq.s	NumInp
	clr.b	LastWasUnary(a5)
NumClr
	clr.b	ReadOut(a5)					;else clear display
	clr.b	DecimalFlag(a5)					; and decimal flag
NumInp	move.b	d0,instr(a5)					;move to temp buffer
	clr.b   instr+1(a5)					;null terminate string
	systrap	StrCat(&ReadOut(a5),&instr(a5))			;concatinate to display
	clr.b	LastWasOp(a5)					;set last input to num
	clr.b	LastWasUnary(a5)				;set last input to num
	clr.b	LastWasMem(a5)
NumDisp	cmp.w	#10,CurrentBase(a5)				;decimal uses floating point
	beq.s	NumDec
	call	InputConversion(&ReadOut(a5),CurrentBase(a5).w) ;Convert to binary in d0.l
	call	UpdateDisplay(d0.l)				;Update display with new value
	bra	MFHEat						;event complete
NumDec
        systrap FplAToF(&fpnTemp(a5),&ReadOut(a5))		;Ascii readout to float in fpnTemp
	call	UpdateDisplay(&fpnTemp(a5))			;Update display with new value
	bra	MFHEat						;event complete
	
NoDigit
	cmp.w   #kidbMC,EventType.data+ctlSelect.controlID(a0)	;Keypress was clear?
	bne.s 	NotMC						;nope, continue
	move.b	#1,LastWasOp(a5)
	move.b	#1,LastWasUnary(a5)
	clr.b	LastWasMem(a5)
	clr.l	FloatType.man+Mem(a5)
	clr.w	FloatType.exp+Mem(a5)
	move.b	#1,FloatType.sign+Mem(a5)
	call	MemDisplay()
        systrap FplAToF(&Op1(a5),&ReadOut(a5))			;
	bra	MFHEat						;event complete
NotMC
	cmp.w   #kidbMR,EventType.data+ctlSelect.controlID(a0)	;Keypress was clear?
	bne 	NotMR						;nope, continue
	clr.b	ReadOut(a5)					;else clear display
	clr.b	DecimalFlag(a5)					; and decimal flag
	clr.b	LastWasOp(a5)
	clr.b	LastWasUnary(a5)
	move.b	#1,LastWasMem(a5)
	cmp.w	#10,CurrentBase(a5)				;decimal?
	beq.s	MFHMRDecDisplay					;yes, decimal display
	move.b	FloatType.sign+Mem(a5),-(a7)
	move.w	FloatType.exp+Mem(a5),-(a7)
	move.l	FloatType.man+Mem(a5),-(a7)
	trap	#15
	dc.w	sysTrapFplFloatToULong				;convert float to unsigned long
	addq.l	#8,a7
	call	UpdateDisplay(d0.l)				;otherwise binary display
	call	InputConversion(&ReadOut(a5),CurrentBase(a5).w) ;convert readout to binary
        systrap FplAToF(&fpnTemp(a5),&ReadOut(a5))		;convert readout to floating point
	bra	MFHEat						;event complete
MFHMRDecDisplay
	call	UpdateDisplay(&Mem(a5))				;update floating display
	call	InputConversion(&ReadOut(a5),CurrentBase(a5).w) ;convert readout to binary
	move.l	d0,fpnTemp(a5)					;save in operand 2
        systrap FplAToF(&fpnTemp(a5),&ReadOut(a5))		;convert readout to floating point
	bra	MFHEat						;event complete
NotMR
	cmp.w   #kidbMS,EventType.data+ctlSelect.controlID(a0)	;Keypress was clear?
	bne.s 	NotMS						;nope, continue
	move.b	#1,LastWasOp(a5)
	move.b	#1,LastWasUnary(a5)
	clr.b	LastWasMem(a5)
	cmp.w	#10,CurrentBase(a5)				;decimal?
	beq.s	MFHMSDec					;yes, decimal display
	call	InputConversion(&ReadOut(a5),CurrentBase(a5).w) ;convert readout to binary
	systrap	FplLongToFloat(&Op1(a5),d0.l)			;put in Mem
MFHMSDec
        systrap FplAToF(&Op1(a5),&ReadOut(a5))			;
MFHMSDisp
	move.b	FloatType.sign+Mem(a5),-(a7)
	move.w	FloatType.exp+Mem(a5),-(a7)
	move.l	FloatType.man+Mem(a5),-(a7)
	move.b	FloatType.sign+Op1(a5),-(a7)
	move.w	FloatType.exp+Op1(a5),-(a7)
	move.l	FloatType.man+Op1(a5),-(a7)
	pea.l	Mem(a5)
	trap	#15
	dc.w	sysTrapFplAdd					;Result = Op1 + Op2
	lea.l	20(a7),a7
	call	MemDisplay()
	bra	MFHEat						;event complete
NotMS
	cmp.w   #kidbclr,EventType.data+ctlSelect.controlID(a0) ;Keypress was clear?
	bne.s 	NotClr						;nope, continue
	call	Initialization()				;initialize calculator
	cmp.w	#10,CurrentBase(a5)				;dec mode?
	bne.s	ClrBin						;no, do binary mode init
	call	UpdateDisplay(&Zero(a5))			;updatedisplay with floating zero
	bra	MFHEat						;event complete
ClrBin	
	call	UpdateDisplay(#0.l)				;update display with binary 0
	bra	MFHEat						;event complete

NotClr  cmp.w   #kidbpercent,EventType.data+ctlSelect.controlID(a0) ;% key?
	bne 	Notpct						;nope, continue
	cmp.w	#10,CurrentBase(a5)				;decimal mode?
	beq.s	PctDec						;yes, do floating percent
;	bOp1=ReadOut*bOp1/100					;formula is different for binary %
	call	InputConversion(&ReadOut(a5),CurrentBase(a5).w) ;Convert display to binary
	call	LongMultiply(d0.l,bOp1(a5).l)			;*Operand 1
	call	LongDivide(d0.l,#100.l)				;/100
	call	UpdateDisplay(d0.l)				;update display with results
	bra	MFHEat						;event complete
PctDec
;	Op1=Op1*ReadOut/100
        systrap FplAToF(&fpnTemp(a5),&ReadOut(a5))		;ReadOut to floating in fpnTemp
	move.b	#$01,-(a7)	 				;this is 			
	move.w	#$ffe7,-(a7)	 				; 100
	move.l	#$c8000000,-(a7) 				;  in floating point
	move.b	FloatType.sign+fpnTemp(a5),-(a7)
	move.w	FloatType.exp+fpnTemp(a5),-(a7)
	move.l	FloatType.man+fpnTemp(a5),-(a7)
	pea.l	fpnTemp(a5)
	trap	#15					
	dc.w	sysTrapFplDiv					;divide readout by 100
	lea.l	20(a7),a7
	move.b	FloatType.sign+fpnTemp(a5),-(a7)
	move.w	FloatType.exp+fpnTemp(a5),-(a7)
	move.l	FloatType.man+fpnTemp(a5),-(a7)
	move.b	FloatType.sign+Op1(a5),-(a7)
	move.w	FloatType.exp+Op1(a5),-(a7)
	move.l	FloatType.man+Op1(a5),-(a7)
	pea.l	fpnTemp(a5)
	trap	#15
	dc.w	sysTrapFplMul					;multiply result by operand 1
	lea.l	20(a7),a7
	call	UpdateDisplay(&fpnTemp(a5))			;update display with result
	bra	MFHEat						;event complete

Notpct  cmp.w   #kidbce,EventType.data+ctlSelect.controlID(a0)  ;Clear Entry key?
	bne.s 	NotCE						;nope, continue
	tst.b	LastWasOp(a5)					;Last input operation?
	bne.s	CEOp						;yes, logic is different
	cmp.w	#10,CurrentBase(a5)				;decimal mode?
	beq.s	CEDec						;yes, clear last decimal entry
	call	UpdateDisplay(bOp1(a5).l)			;update display with last binary entry
	move.b	#0,ReadOut(a5)					;clear display
	bra	MFHEat						;event complete
CEDec	
	call	UpdateDisplay(&Op1(a5))				;update disp with last float entry
	move.b	#0,ReadOut(a5)					;clear display	
	bra	MFHEat						;event complete
CEOp	clr.b	LastOp(a5)					;no last op
	clr.b	NumOps(a5)					;no ops yet
	cmp.w	#10,CurrentBase(a5)				;decimal mode?
	bne.s	CEOpBin						;no, it's binary mode
	call	UpdateDisplay(&Op1(a5))				;update display with last float entry
	bra	MFHEat						;event complete
CEOpBin 
	call	UpdateDisplay(bOp1(a5).l)			;update display with last bin 
	bra	MFHEat						;event complete

NotCE	cmp.w	#kidblshift,EventType.data+ctlSelect.controlID(a0) ; left shift binary operation?
	bne.s	NotLSft						;nope, continue
	move.b	#1,LastWasUnary(a5)
	call	InputConversion(&ReadOut(a5),CurrentBase(a5).w) ;convert Readout to binary
	lsl.l	#1,d0						;shift left 1 bit
	move.l	d0,bOp1(a5)
	jsr	BinOpBase10End(pc)
	bra	MFHDecBinDisplay

NotLSft	cmp.w	#kidbrshift,EventType.data+ctlSelect.controlID(a0) ;right shift binary operation?
	bne.s	NotRSft						;nope, continue
	move.b	#1,LastWasUnary(a5)
	call	InputConversion(&ReadOut(a5),CurrentBase(a5).w) ;convert ReadOut to binary
	lsr.l	#1,d0						;shift right 1 bit
	move.l	d0,bOp1(a5)
	jsr	BinOpBase10End(pc)
	bra	MFHDecBinDisplay

NotRSft	cmp.w	#kidblrotate,EventType.data+ctlSelect.controlID(a0) ; left rotate binary operation?
	bne.s	NotLrot						;nope, continue
	cmp.w	#10,CurrentBase(a5)				;decimal?
	beq	beep						;invalid keypress if decimal mode
	move.b	#1,LastWasUnary(a5)
	call	InputConversion(&ReadOut(a5),CurrentBase(a5).w) ;convert Readout to binary
	rol.l	#1,d0						;shift left 1 bit
	call	UpdateDisplay(d0.l)				;update display with new results
	bra	MFHEat						;event complete

NotLrot	cmp.w	#kidbrrotate,EventType.data+ctlSelect.controlID(a0) ;right rotate binary operation?
	bne.s	NotRrot						;nope, continue
	cmp.w	#10,CurrentBase(a5)				;decimal?
	beq	beep						;invalid keypress if decimal
	move.b	#1,LastWasUnary(a5)
	call	InputConversion(&ReadOut(a5),CurrentBase(a5).w) ;convert ReadOut to binary
	ror.l	#1,d0						;shift right 1 bit
	call	UpdateDisplay(d0.l)				;Update Display with new value
	bra	MFHEat						;event complete

NotRrot	cmp.w	#kidbneg,EventType.data+ctlSelect.controlID(a0) ;Negate binary operation?
	bne.s	NotNeg						;nope, continue
	cmp.w	#10,CurrentBase(a5)				;decimal?
	beq	NegDec						;negate decimal
	call	InputConversion(&ReadOut(a5),CurrentBase(a5).w) ;convert ReadOut to binary
	neg.l	d0						;negate 
	call	UpdateDisplay(d0.l)				;update display with results
	bra	MFHEat						;event complete
NegDec  
        systrap FplAToF(&fpnTemp(a5),&ReadOut(a5))		;convert display to fpnTemp
	move.b	#$ff,-(a7)	 				;this is 			
	move.w	#$ffe1,-(a7)	 				; -1
	move.l	#$80000000,-(a7) 				;  in floating point
	move.b	FloatType.sign+fpnTemp(a5),-(a7)
	move.w	FloatType.exp+fpnTemp(a5),-(a7)
	move.l	FloatType.man+fpnTemp(a5),-(a7)
	pea.l	fpnTemp(a5)
	trap	#15					
	dc.w	sysTrapFplMul					;multiply by -1
	lea.l	20(a7),a7
	call	UpdateDisplay(&fpnTemp(a5))
	bra	MFHEat

NotNeg	cmp.w	#kidbnot,EventType.data+ctlSelect.controlID(a0) ;Not (1's complement) operation?
	bne.s	NotNot						;nope, continue
	move.b	#1,LastWasUnary(a5)
binary
	call	InputConversion(&ReadOut(a5),CurrentBase(a5).w)	;convert display to binary
	not.l	d0						;not
	move.l	d0,bOp1(a5)
	jsr	BinOpBase10End(pc)
	bra	MFHDecBinDisplay

NotNot	cmp.w	#kidbsetbase,EventType.data+ctlSelect.controlID(a0) ;set base?
	bne	Oper						;nope, continue
	systrap	FrmInitForm(#kidfBase.w)			;init dialog
	move.l	a0,a3						;put FormPtr in a safe place
	systrap	FrmDoDialog(a3.l)				;do it
        systrap FrmGetObjectIndex(a3.l, #kidcbase.w)		;get field index
        systrap FrmGetObjectPtr(a3.l,d0.w)			;get pointer
        systrap FldGetTextPtr(a0.l)				;get address of text
	cmp.l	#0,a0						;is it null?
	beq	DlgDel						;yes, user didn't enter anything
	tst.b	(a0)						;is the string null?
	beq	DlgDel						;yes, same thing
	systrap	StrAToI(a0.l)					;convert the string to integer
	cmp.w	#2,d0						;base must be
	blt	BadBase						; between 2
	cmp.w	#16,d0						;  and 16
	bgt	BadBase
	move.w	CurrentBase(a5),d1				;save old base
	move.w	d0,CurrentBase(a5)				;save new base
	call	ConvertBaseToBase(d1.w,d0.w)			;Convert operands and display to new base
	systrap	FrmGetActiveForm()				;get main form
	move.l	a0,a4						;save in a4
	systrap	FrmGetControlGroupSelection(a4.l,#1.b)		;get current selection 
	systrap	FrmGetObjectPtr(a4.l,d0.w)			;get object pointer
	systrap	CtlSetValue(a0.l,#0.w)				;reset button
	cmp.w	#10,CurrentBase(a5)				;now set the proper button
	bne.s	Base2
	systrap	FrmSetControlGroupSelection(a4.l,#1.b,#10.w)
	bra.s	DlgDel
Base2	cmp.w	#2,CurrentBase(a5)
	bne.s	Base8
	systrap	FrmSetControlGroupSelection(a4.l,#1.b,#2.w)
	bra.s	DlgDel
Base8	cmp.w	#8,CurrentBase(a5)
	bne.s	Base16
	systrap	FrmSetControlGroupSelection(a4.l,#1.b,#8.w) 
	bra.s	DlgDel
Base16	cmp.w	#16,CurrentBase(a5)
	bne.s	DlgDel
	systrap	FrmSetControlGroupSelection(a4.l,#1.b,#16.w) 
DlgDel
	systrap	FrmDeleteForm(a3.l)				;delete form
	bra	NumDisp
BadBase
        systrap FrmAlert(#kidrBaseError.w)			;tell user base must be 2-16
	bra	MFHEat

Oper	tst.b	LastWasOp(a5)					;LastWasOp was num?
	bne.s	LNotNum
	addq.b	#1,NumOps(a5)					;increments number of args
LNotNum	cmp.b	#1,NumOps(a5)					;num args = 1?
	bne.s	ChkTwo						;no, then continue
	cmp.w	#10,CurrentBase(a5)				;decimal mode?
	beq.s	CtF						;yes, dec mode uses floating point
	call	InputConversion(&ReadOut(a5),CurrentBase(a5).w) ;otherwise convert readout to binary
	move.l	d0,bOp1(a5)					;save in operand 1
	bra	MFHDone						;done, go update display
CtF
        systrap FplAToF(&Op1(a5),&ReadOut(a5))			;op1 = last value entered
	bra	MFHDone						;Go update display

ChkTwo	cmp.b	#2,NumOps(a5)					;two operands?
	bne	MFHOpX						;no, then were done
	cmp.w	#10,CurrentBase(a5)				;Dec mode?
	beq.s	CtF2						;yes, dec uses floating point
	call	InputConversion(&ReadOut(a5),CurrentBase(a5).w) ;convert readout to binary
	move.l	d0,bOp2(a5)					;save in operand 2
	bra.s	CtF2x						;and continue
CtF2
        systrap FplAToF(&Op2(a5),&ReadOut(a5))			;convert readout to floating point
CtF2x	cmp.b	#kidband,LastOp(a5)				;last operation and?
	bne.s	NotAnd						;no, try next
	jsr	BinOpBase10Begin(pc)
	move.l	bOp1(a5),d0					;get operand 1
	and.l	bOp2(a5),d0					;and with operand 2
	move.l	d0,bOp1(a5)					;save result
	jsr	BinOpBase10End(pc)
	bra	MFHOpX						;complete

NotAnd	cmp.b	#kidbor,LastOp(a5)				;last operation 'or'
	bne.s	NotOr						;no, try next
	jsr	BinOpBase10Begin(pc)
	move.l	bOp1(a5),d0					;get operand 1
	or.l	bOp2(a5),d0					;or with operand 2
	move.l	d0,bOp1(a5)					;save result
	jsr	BinOpBase10End(pc)
	bra	MFHOpX						;complete

NotOr	cmp.b	#kidbxor,LastOp(a5)				;last operation 'xor'
	bne.s	NotXor						;no, try next
	jsr	BinOpBase10Begin(pc)
	move.l	bOp1(a5),d0					;get operand 1
	move.l	bOp2(a5),d1					;get operand 2
	eor.l	d1,d0						;Xor 
	move.l	d0,bOp1(a5)					;save result
	jsr	BinOpBase10End(pc)
	bra	MFHOpX						;complete

NotXor	cmp.b	#kidbmod,LastOp(a5)				;last operation 'mod'
	bne	NotMod						;no, try next
	cmp.w	#10,CurrentBase(a5)				;deciaml?
	bne	ModBinary					;no, do in binary
	call	CheckDBZ(&Op2(a5))				;are we going to divide by zero?
	tst.b	d0						;d0 not zero then error
	bne	MFHOpX		
	move.b	FloatType.sign+Op2(a5),-(a7)
	move.w	FloatType.exp+Op2(a5),-(a7)
	move.l	FloatType.man+Op2(a5),-(a7)
	move.b	FloatType.sign+Op1(a5),-(a7)
	move.w	FloatType.exp+Op1(a5),-(a7)
	move.l	FloatType.man+Op1(a5),-(a7)
	pea.l	fpnTemp(a5)
	trap	#15
	dc.w	sysTrapFplDiv					;fpnTemp = Op1 / Op2
	lea.l	20(a7),a7
	move.b	FloatType.sign+fpnTemp(a5),-(a7)
	move.w	FloatType.exp+fpnTemp(a5),-(a7)
	move.l	FloatType.man+fpnTemp(a5),-(a7)
	trap	#15
	dc.w	sysTrapFplFloatToLong				;d0.l = int(fpnTemp)
	addq.l	#8,a7
	systrap	FplLongToFloat(&Op1(a5),d0.l)			;put in in Op1
	move.b	FloatType.sign+Op1(a5),-(a7)
	move.w	FloatType.exp+Op1(a5),-(a7)
	move.l	FloatType.man+Op1(a5),-(a7)
	move.b	FloatType.sign+fpnTemp(a5),-(a7)
	move.w	FloatType.exp+fpnTemp(a5),-(a7)
	move.l	FloatType.man+fpnTemp(a5),-(a7)
	pea.l	Op1(a5)
	trap	#15
	dc.w	sysTrapFplSub					;Op1 = fractional porition of Op2
	lea.l	20(a7),a7
	move.b	FloatType.sign+Op2(a5),-(a7)
	move.w	FloatType.exp+Op2(a5),-(a7)
	move.l	FloatType.man+Op2(a5),-(a7)
	move.b	FloatType.sign+Op1(a5),-(a7)
	move.w	FloatType.exp+Op1(a5),-(a7)
	move.l	FloatType.man+Op1(a5),-(a7)
	pea.l	Op1(a5)
	trap	#15
	dc.w	sysTrapFplMul					;Result = Op1 mod Op2
	lea.l	20(a7),a7
	bra	MFHOpX
ModBinary
	tst.l	bOp2(a5)					;divide by zero?
	bne.s	ModBinaryOK					;no, continue
        systrap FrmAlert(#kidrDivideZero.w)			;issue alert
	bra	MFHEat						
ModBinaryOK
	call	LongRemainder(bOp1(a5).l,bOp2(a5).l)		;d0.l = bOp1 / bOp2
	move.l	d0,bOp1(a5)					;save result
	bra	MFHOpX

NotMod	cmp.b   #kidbadd,LastOp(a5)				;last operation add?
	bne.s 	NotAdd						;no, try next
	cmp.w	#10,CurrentBase(a5)				;are we decimal mode?
	bne.s	AddBinary					;no, then binary mode
	move.b	FloatType.sign+Op1(a5),-(a7)
	move.w	FloatType.exp+Op1(a5),-(a7)
	move.l	FloatType.man+Op1(a5),-(a7)
	move.b	FloatType.sign+Op2(a5),-(a7)
	move.w	FloatType.exp+Op2(a5),-(a7)
	move.l	FloatType.man+Op2(a5),-(a7)
	pea.l	Op1(a5)
	trap	#15
	dc.w	sysTrapFplAdd					;Result = Op1 + Op2
	lea.l	20(a7),a7
	bra	MFHOpX
AddBinary
	move.l	bOp1(a5),d0					;get op1	
	add.l	bOp2(a5),d0					;add in op2
	move.l	d0,bOp1(a5)					;save result
	bra	MFHOpX						;continue with binary add/sub/mul/div

NotAdd	cmp.b   #kidbsub,LastOp(a5)				;subtract?
	bne.s	NotSub						;no, try next
	cmp.w	#10,CurrentBase(a5)				;decimal mode?
	bne.s	SubBinary					;no do in binary
	move.b	FloatType.sign+Op2(a5),-(a7)
	move.w	FloatType.exp+Op2(a5),-(a7)
	move.l	FloatType.man+Op2(a5),-(a7)
	move.b	FloatType.sign+Op1(a5),-(a7)
	move.w	FloatType.exp+Op1(a5),-(a7)
	move.l	FloatType.man+Op1(a5),-(a7)
	pea.l	Op1(a5)
	trap	#15
	dc.w	sysTrapFplSub					;Result = Op1 - Op2
	lea.l	20(a7),a7
	bra	MFHOpX
SubBinary
	move.l	bOp1(a5),d0					;get op1
	sub.l	bOp2(a5),d0					;add in op2
	move.l	d0,bOp1(a5)					;save result
	bra	MFHOpX

NotSub	cmp.b   #kidbmul,LastOp(a5)				;last operation multiply?
	bne.s	NotMul						;no, continue
	cmp.w	#10,CurrentBase(a5)				;decimal mode?
	bne.s	MulBinary					;no, do in binary
	move.b	FloatType.sign+Op2(a5),-(a7)
	move.w	FloatType.exp+Op2(a5),-(a7)
	move.l	FloatType.man+Op2(a5),-(a7)
	move.b	FloatType.sign+Op1(a5),-(a7)
	move.w	FloatType.exp+Op1(a5),-(a7)
	move.l	FloatType.man+Op1(a5),-(a7)
	pea.l	Op1(a5)
	trap	#15
	dc.w	sysTrapFplMul					;Result = Op1 * Op2
	lea.l	20(a7),a7
	bra	MFHOpX
MulBinary
	call	LongMultiply(bOp1(a5).l,bOp2(a5).l)		;d0.l = bOp1 * bOp2
	move.l	d0,bOp1(a5)					;save result
	bra	MFHOpX

NotMul	cmp.b   #kidbdiv,LastOp(a5)				;last operation divide?
	bne.s	NotDiv						;no, try next
	cmp.w	#10,CurrentBase(a5)				;deciaml?
	bne.s	DivBinary					;no, do in binary
	call	CheckDBZ(&Op2(a5))				;are we going to divide by zero?
	tst.b	d0						;d0 not zero then error
	bne	MFHOpX		
	move.b	FloatType.sign+Op2(a5),-(a7)
	move.w	FloatType.exp+Op2(a5),-(a7)
	move.l	FloatType.man+Op2(a5),-(a7)
	move.b	FloatType.sign+Op1(a5),-(a7)
	move.w	FloatType.exp+Op1(a5),-(a7)
	move.l	FloatType.man+Op1(a5),-(a7)
	pea.l	Op1(a5)
	trap	#15
	dc.w	sysTrapFplDiv					;Result = Op1 / Op2
	lea.l	20(a7),a7
	bra	MFHOpX
DivBinary
	tst.l	bOp2(a5)					;divide by zero?
	bne.s	DivBinaryOK					;no, continue
        systrap FrmAlert(#kidrDivideZero.w)			;issue alert
	bra	MFHEat						
DivBinaryOK
	call	LongDivide(bOp1(a5).l,bOp2(a5).l)		;d0.l = bOp1 / bOp2
	move.l	d0,bOp1(a5)					;save result
	bra	MFHOpX

NotDiv	cmp.b   #kidbequal,LastOp(a5)				;last operation '=' key?
	bne	MFHNotHandled					;no, last operation, dont handle it
	cmp.w	#10,CurrentBase(a5)				;decimal?
	bne.s	EqualBinary					;no, then binary mode
	move.l	FloatType.man+Op2(a5),FloatType.man+Op1(a5)	;move 
	move.w	FloatType.exp+Op2(a5),FloatType.exp+Op1(a5)	; Op2
	move.b	FloatType.sign+Op2(a5),FloatType.sign+Op1(a5)	;  to Op1
	bra	MFHOpX	
EqualBinary
	move.l	bOp2(a5),bOp1(a5)				;move binary Op2 to Op1

MFHOpX	move.b	#1,NumOps(a5)
MFHDone movea.l pevt(a6),a0					;a0 = pevt
	move.w	EventType.data+ctlSelect.controlID(a0),d0	;get event
	move.b	d0,LastOp(a5)					;save in last operation field
	move.b	#1,LastWasOp(a5)				;set to last was operation
	clr.b	LastWasMem(a5)
MFHDecBinDisplay
	cmp.w	#10,CurrentBase(a5)				;decimal?
	beq.s	MFHDecDisplay					;yes, decimal display
	call	UpdateDisplay(bOp1(a5).l)			;otherwise binary display
	bra.s	MFHEat						;event complete
MFHDecDisplay
	call	UpdateDisplay(&Op1(a5))				;update floating display
MFHEat	moveq.l	#1,d0						;set d0 to say we took event
	bra.s	MFHReturn					;and return
MFHNotHandled	
        clr.b   d0						;set d0 to say event not handled
MFHReturn
        movem.l (a7)+,a3-a4/d2					;restore regs
endproc

BinOpBase10Begin:
	cmp.w	#10,CurrentBase(a5)		;decimal?
	bne	BOB10B1				;no, skip routine
	move.b	FloatType.sign+Op1(a5),-(a7)
	move.w	FloatType.exp+Op1(a5),-(a7)
	move.l	FloatType.man+Op1(a5),-(a7)
	trap	#15
	dc.w	sysTrapFplFloatToULong		;convert float to unsigned long
	addq.l	#8,a7
	move.l	d0,bOp1(a5)			;save converted binary operand
	move.b	FloatType.sign+Op2(a5),-(a7)
	move.w	FloatType.exp+Op2(a5),-(a7)
	move.l	FloatType.man+Op2(a5),-(a7)
	trap	#15
	dc.w	sysTrapFplFloatToULong		;convert float to unsigned long
	addq.l	#8,a7
	move.l	d0,bOp2(a5)			;save converted binary
BOB10B1
	rts

BinOpBase10End:
	cmp.w	#10,CurrentBase(a5)		;decimal?
	bne	BOB10E1				;no, skip routine
	systrap	FplLongToFloat(&Op1(a5),bOp1(a5).l)	;convert Op1
	systrap	FplLongToFloat(&Op2(a5),bOp2(a5).l)	;convert Op2
BOB10E1
	rts

proc ConvertBaseToBase(OldBase.w,NewBase.w)
beginproc
	movem.l	a3,-(a7)					;save reg
	call	InputConversion(&ReadOut(a5),OldBase(a6).w) 	;Convert input to bin in d0
	cmp.w	#10,NewBase(a6)					;is new base decimal?
	bne.s	BBOutputBinary	
	systrap	FplLongToFloat(&fpnTemp(a5),d0.l)		;convert float
	call	Conv(&fpnTemp(a5),&ReadOut(a5))
	bra	BBExchangeOperands
BBOutputBinary
	clr.l	d1						;outputconversion takes a long
	move.w	NewBase(a6),d1					;get new base in d1.w
	call	OutputConversion(d0.l,d1.l,&TempBuffer(a5),#40.l) ;Convert to Displayable in Temp
	systrap	StrCopy(&ReadOut(a5),a3.l)			;copy result to Readout
BBExchangeOperands
	cmp.w	#10,OldBase(a6)					;is old base decimal?
	beq.s	FloatToBin					;yes, then possible float->bin
	cmp.w	#10,NewBase(a6)					;is new base decimal?
	bne	ExchangeExit					;neither to or from decimal, staying in bin
	; exchange from binary to float
	systrap	FplLongToFloat(&Op1(a5),bOp1(a5).l)		;convert Op1
	systrap	FplLongToFloat(&Op2(a5),bOp2(a5).l)		;convert Op2
	bra	ExchangeExit					;done
FloatToBin
	cmp.w	#10,NewBase(a6)					;is new base decimal also?
	beq.s	ExchangeExit					;yes,no conversion
	move.b	FloatType.sign+Op1(a5),-(a7)
	move.w	FloatType.exp+Op1(a5),-(a7)
	move.l	FloatType.man+Op1(a5),-(a7)
	trap	#15
	dc.w	sysTrapFplFloatToULong				;convert float to unsigned long
	addq.l	#8,a7
	move.l	d0,bOp1(a5)					;save converted binary operand
	move.b	FloatType.sign+Op2(a5),-(a7)
	move.w	FloatType.exp+Op2(a5),-(a7)
	move.l	FloatType.man+Op2(a5),-(a7)
	trap	#15
	dc.w	sysTrapFplFloatToULong				;convert float to unsigned long
	addq.l	#8,a7
	move.l	d0,bOp2(a5)					;save converted binary operand
ExchangeExit
	movem.l	(a7)+,a3					;restore callers reg
endproc

proc CheckDBZ(fpn.l)						;Check if divide by zero
beginproc
	movea.l	fpn(a6),a0	
	clr.l	d0						;assume not zero
	cmp.l	#0,FloatType.man(a0)				;compare floating
	bne.s	DBZx						; operand
	cmp.w	#0,FloatType.exp(a0)				;  to 
	bne.s	DBZx						;   floating
	cmp.b	#1,FloatType.sign(a0)				;    point
	bne.s	DBZx						;      zero
        systrap FrmAlert(#kidrDivideZero.w)			;and display error if it is
	move.l	#1,d0						;return true, would DBZ
DBZx
endproc

proc MemDisplay()
local rect.RectangleType
beginproc
	cmp.l	#0,FloatType.man+Mem(a5)
	beq.s	.2
.1
	systrap WinDrawChars(&memchr(a5).l,#1.w,#12.w,#18.w)	;draw the character
	bra	.3						;event complete
.2
	move.w	#12,rect.topLeft.x(a6)
	move.w	#18,rect.topLeft.y(a6)
	move.w	#8,rect.extent.x(a6)
	move.w	#10,rect.extent.y(a6)
	systrap	WinEraseRectangle(&rect(a6),#0.w)		;draw rectangle around
.3
endproc

proc UpdateDisplay(fpn.l)
local rect.RectangleType
beginproc
	movem.l	d2-d3/a3,-(a7)
	cmp.w	#10,CurrentBase(a5)				;decimal display mode?
	beq.s	UDDec						;yes, continue
	clr.l	d1						;OutputConverion takes a long
	move.w	CurrentBase(a5),d1				;move in currentbase
	call	OutputConversion(fpn(a6).l,d1.l,&TempBuffer(a5),#40.l) ;convert new value to display
	systrap	StrCopy(&ReadOut(a5),a3.l)			;copy to readout
	lea	ReadOut(a5),a3					;set address of readout
	bra.s	UDDisp						;continue
UDDec	lea	ReadOut(a5),a3					;set address of readout in a3
	tst.b	DecimalFlag(a5)					;user previously pressed dec pt.
	beq.s	UDNDot						;no, continue
	systrap	StrLen(a3.l)					;get length of readout
	subq.l	#1,d0						;-1
	cmp.b	#'0',(a3,d0)					;pressing .00000
	beq.s	UDDisp						;yes, display asis
	cmp.b	#'.',(a3,d0)					;pressing .
	beq.s	UDDisp						;yes, display asis
UDNDot
	call	Conv(fpn(a6).l,a3.l)				;otherwise, convert float
UDDisp	
	systrap	StrCopy(&TempBuffer(a5),a3.l)			;copy new display to temp
	move.w	#9,rect.topLeft.x(a6)
	move.w	#7,rect.topLeft.y(a6)
	move.w	#132,rect.extent.x(a6)
	move.w	#20,rect.extent.y(a6)
	systrap	WinEraseRectangle(&rect(a6),#0.w)		;draw rectangle around display
	lea	TempBuffer(a5),a3				;address of temp
	move.b	#0,14(a3)					;max chars, = 14
	systrap	StrLen(a3.l)					;get string length of display
	move.l	d0,d3						;save string length for later
	lsl.l	#3,d0						;*8 for character width
	add.l	d3,d0						;+1 spacer for each char
	move.l	#137,d2						;ending position of display
	sub.l	d0,d2						;d2 holds starting position 
DspLoop	move.b	(a3)+,d0					;move next char to d0
	ext.w	d0						;make word
	call    DrawBitmap(d0.w,d2.w,#10.w)			;draw a char
	add.l	#9,d2						;increment xpos
	subq.l	#1,d3						;dec length
	bne.s	DspLoop						;continue till input exausted	
	tst.b	LastOp(a5)					;any pending operations?
	beq.s	UDDone						;nope, done
	cmp.b	#kidbdiv,LastOp(a5)				;announce +-*/ ops
	bgt.s	ChkOp2						;and only thouse ops
	systrap	FntSetFont(#stdFont.b)				;set font to standard
	clr.l	d0						;clear d0
	move.b	LastOp(a5),d0					;get last op in d0
	lea.l	opTable(a5,d0),a3				;convert to a character
	systrap WinDrawChars(a3.l,#1.w,#12.w,#6.w)		;draw the character
	bra.s	UDDone						;and done
ChkOp2	cmp.b	#kidbmod,LastOp(a5)				;is this 
	bgt.s	UDDone						;  AND, OR
	cmp.b	#kidband,LastOp(a5)				;    or XOR
	blt.s	UDDone						;      operation?
	systrap	FntSetFont(#stdFont.b)				;set font to standard
	clr.l	d0						;clear d0
	move.b	LastOp(a5),d0					;get last op
	sub.l	#kidband-OPSBASE,d0				;op relative to OPSBASE
	lsl.l	#2,d0						;*4 points to string
	lea.l	opTable2(a5,d0),a3				;get address of string start
	systrap WinDrawChars(a3.l,#3.w,#12.w,#6.w)		;draw string
UDDone	clr.b	DecimalFlag(a5)					;clear decimal flag
	lea	ReadOut(a5),a0					;address of readout
ChkDec	tst.b	(a0)						;end?
	beq.s	DecChkX						;yes, done
	cmp.b	#'.',(a0)+					;do we have a dec pt?		
	bne.s	ChkDec						;no, continue
	move.b	#1,DecimalFlag(a5)				;yes, set result has dec pt
DecChkX
	call	MemDisplay()
	movem.l	(a7)+,d2-d3/a3					;restore regs
endproc

proc InputConversion(pInput.l,iBase.w)
local InputResult.l
local NegFlag.w
beginproc
	clr.l	InputResult(a6)					;clear input value
	movea.l	pInput(a6),a3					;input pointer
	clr.b	NegFlag(a6)					;assume positive
	cmp.b	#'-',(a3)					;neg?
	bne.s	ICLoop	
	move.b	(a3)+,NegFlag(a6)					;set negative
ICLoop	tst.b	(a3)						;input exausted?
	beq.s	ICDone						;yes, input conversion done
	cmp.b	#'.',(a3)					;stop on .
	beq.s	ICDone		
	clr	d1		
	move.w	iBase(a6),d1					
	subq.w	#1,d1						;input base-1 in d1
	move.l	InputResult(a6),d0				
ICMul	tst.w	d1						;InputResult = InputResult * iBase
	beq.s	ICMulx						
	add.l	d0,InputResult(a6)
	subq.w	#1,d1
	bra.s	ICMul
ICMulx	clr.l	d0
	cmp.b	#'9',(a3)					;pInput <= '9'
	bgt.s	ICAlpha
	move.b	(a3)+,d0
	and.b	#$0f,d0
	add.l	d0,InputResult(a6)				;InputResult += (pInput&0x0f)
	bra.s	ICLoop
ICAlpha	move.b	(a3)+,d0
	and.b	#$0f,d0
	add.b	#9,d0
	add.l	d0,InputResult(a6)				;InputResult += (pInput&0x0f)+9
	bra	ICLoop
ICDone
	move.l	InputResult(a6),d0				;d0 is result
	tst.b	NegFlag(a6)
	beq.s	ICx
	neg.l	d0						;negate result
ICx
endproc

proc OutputConversion(input.l,oBase.l,pOutput.l,len.l)
beginproc
	movea.l	pOutput(a6),a3					;pointer to output buffer
	move.l	len(a6),d0					;get length of output buffer
	lea.l	(a3,d0),a3					;point to end
	move.b	#0,-(a3)					;null terminator
OCLoop	
	call	LongRemainder(input(a6).l,oBase(a6).l)		;get remainder of input/oBase
	cmp.w	#9,d0						;if > 9
	bgt.s	OCAlpha						;then A-F form
	or.b	#$30,d0						;make char
	move.b	d0,-(a3)					;*output |= 0x30
	bra.s	OCLoopx
OCAlpha	sub.b	#10,d0
	move.b	alpha(a5,d0),-(a3)				;*output = alpha[*output-10]
OCLoopx	
	call	LongDivide(input(a6).l,oBase(a6).l)		;input = input/oBase
	move.l	d0,input(a6)
	bgt	OCLoop						;while(input)
endproc


proc LongRemainder(dividend.l,divisor.l)
beginproc
     	call	LongDivide(dividend(a6).l,divisor(a6).l)
     	move.l	divisor(a6),d1
     	call	LongMultiply(d0.l,d1.l)
     	move.l	dividend(a6),d1
     	sub.l	d0,d1
     	move.l	d1,d0
endproc

proc LongMultiply(arg1.l,arg2.l)
beginproc
  	move.w	arg1(a6),d0
     	mulu.w	arg2+2(a6),d0
     	move.w	arg1+2(a6),d1
     	mulu.w	arg2(a6),d1
     	add.w	d1,d0
     	swap	d0
     	clr.w	d0
     	move.w	arg1+2(a6),d1
     	mulu.w	arg2+2(a6),d1
     	add.l	d1,d0
endproc

proc LongDivide(dividend.l,divisor.l)
beginproc
	move.l	d2,-(a7)					;save reg used
     	move.l	divisor(a6),d1					;divisor
     	move.l	dividend(a6),d0					;dividend	
     	cmpi.l	#$10000,d1					;anticipate overflow?
     	bcc	Ovfl						;yes, handle it
     	move.l	d0,d2						;dividend to d2
     	clr.w	d2						;upper divide first
     	swap	d2						;upper to lower
     	divu.w	d1,d2						;divide by divisor result to d2
     	move.w	d2,d0						;move result to d0
     	swap	d0						;quotient to upper, remainder to lower
     	move.w	dividend+2(a6),d2				;get lower portion of dividend
     	divu.w	d1,d2						;divide it
     	move.w	d2,d0						;result to d0
     	bra	DivExit						;return					
Ovfl  	move.l	d1,d2						;save divisor in d2
Shft  	lsr.l	#1,d1						;right shift divisor
     	lsr.l	#1,d0						;and dividend 
     	cmpi.l	#$10000,d1					;until no overflow exists
     	bcc	Shft						;no overflow condition
     	divu.w	d1,d0						;dividend/divisor -> remainder|quotient)
     	andi.l	#$ffff,d0					;isolate quotient
  	move.l	d2,d1						;saved divisor to d1
     	mulu.w	d0,d1						;quotient * divisor lower -> d1
     	swap	d2						;divisor upper to lower
     	mulu.w	d0,d2						;quotient * divisor upper -> d2
     	swap	d2						;test
     	btst.l	#0,d2						;negative?
     	bne	DecRslt						;no, then done
     	add.l	d2,d1				
     	bcs	DecRslt
     	cmp.l	dividend(a6),d1
     	bls	DivExit
DecRslt subq.l	#1,d0
DivExit	move.l	(a7)+,d2					;restore used reg	
endproc


proc DrawBitmap(resource.w,x.w,y.w)
beginproc
	movem.l	a3/d2-d3,-(a7)					;save regs used
	move.l  #'Tbmp',d0					;bitmap resource id
   	systrap DmGetResource(d0.l, resource(a6).w)		;get the resource
	cmp.l	#0,a0						;success?
	beq.s   DrawBitmapError					;programmer screwed up
	move.l	a0,a3						;move to a3
	systrap MemHandleLock(a0.l)				;lock resource
	cmp.l	#0,a0						;did it happen?
	beq.s	DrawBitmapError					;programmer screwed up
	systrap WinDrawBitmap(a0.l,x(a6).w,y(a6).w)		;draw the bitmap at the specified coord
	systrap MemHandleUnlock(a3.l)				;unlock handle
	systrap DmReleaseResource(a3.l)				;release resource
DrawBitmapError
	movem.l	(a7)+,a3/d2-d3					;restore reg
endproc


proc Initialization()
beginproc
	clr.b	ReadOut(a5)					;null string readout
	clr.b	DecimalFlag(a5)					;clear decimal flag
	clr.b	LastOp(a5)					;no last op
	clr.b	NumOps(a5)					;no ops yet
	clr.b	LastWasOp(a5)					; = "none"
	clr.b	LastWasUnary(a5)					; = "none"
	clr.b	LastWasMem(a5)
	clr.l	FloatType.man+Op1(a5)
	clr.w	FloatType.exp+Op1(a5)
	move.b	#1,FloatType.sign+Op1(a5)			;set all floats to zero
	clr.l	FloatType.man+Op2(a5)
	clr.w	FloatType.exp+Op2(a5)
	move.b	#1,FloatType.sign+Op2(a5)
	clr.l	bOp1(a5)					;clear binary ops
	clr.l	bOp2(a5)
endproc

proc Conv(pFpn.l,pResult.l)
local expNegFlag.w
beginproc
	movem.l	a3-a4,-(a7)
	move.l	pResult(a6),-(a7)
	movea.l	pFpn(a6),a0
	move.b	FloatType.sign(a0),-(a7)
	move.w	FloatType.exp(a0),-(a7)
	move.l	FloatType.man(a0),-(a7)
	trap 	#15
	dc.w	sysTrapFplFToA
	lea.l	12(a7),a7
	tst.l	d0
	beq.s	fplOK
        systrap FrmAlert(#kidrFPConvError.w)
	bra	ConvXit
;The result will be [-]x[.]yyyyyyy[e[-]zz] or 0
fplOK	movea.l	pResult(a6),a3					;a3 -> source field
	lea	TempBuffer(a5),a4				;a4 -> destination string
	cmp.b	#'0',(a3)					;result zero?
	beq	CnvZero						;yes, make sure dec pt ok
	clr.b	expNegFlag(a6)					;clear exponent flag
	cmp.b	#'-',(a3)					;neg mantissa result?
	bne 	notNeg
	move.b	(a3)+,(a4)+					;negative result
notNeg  cmp.b	#'-',10(a3)					;is exponent negative?
	bne	expPos
	move.b	#'-',expNegFlag(a6)				;save sign of exponent
	pea	11(a3)
	bra.s	doExpCv
expPos	pea	10(a3)
doExpCv trap	#15						;convert exponent to int in d0
	dc.w	sysTrapStrAToI
	addq.l	#4,a7
	cmp.l	#7,d0						;only convert +/- 7 exponents
	bgt	ConvXit						;exponent out of range, just display
manPos	tst.b	expNegFlag(a6)					;exponent neg?
	beq.s	expIsP
	move.b	#'0',(a4)+					;put in the decimal point
	move.b	#'.',(a4)+
	subq.l	#1,d0						;exp-1 digits left 
neLoop	tst.l	d0			
	beq.s	moveMan
	move.b	#'0',(a4)+					;move leading zeros
	subq.l	#1,d0
	bra.s	neLoop
expIsP  cmp.b	#'.',(a3)					;don't move the dec pt in the input stream
	bne.s	expIsPm
	lea	1(a3),a3					;inc input stream ptr
	bra.s	expIsP						;continue...
expIsPm	move.b	(a3)+,(a4)+					;move input stream to output
	dbeq	d0,expIsP					;continue
	move.b	#'.',(a4)+					;here's where the dp goes
moveMan	cmp.b	#'.',(a3)					;is it the dec pt on input?
	bne.s	moveMn2						;yes, then skip
	lea	1(a3),a3
	bra.s	moveMan
moveMn2	cmp.b	#'e',(a3)					;end?
	beq.s	moveEnd
	move.b	(a3)+,(a4)+					;move input to output
	bra.s	moveMan
moveEnd	cmp.b	#'0',-(a4)					;Righthand zero suppress
	beq.s	moveEnd
	cmp.b	#'.',(a4)					;if x.0 form
	bne.s	EndDgt
	tst.b	DecimalFlag(a5)					;should we display the dot?
	bne.s	EndDgt
	clr.b	(a4)						;c-string type terminate string
	bra.s	Reslt
CnvZero	tst.b	DecimalFlag(a5)					;decimal entered?
	beq.s	ConvXit						;no, continue
	move.b	#'0',(a4)+
	move.b	#'.',(a4)
EndDgt	move.b	#0,1(a4)	
Reslt	movea.l	pResult(a6),a0
	systrap	StrCopy(a0.l,&TempBuffer(a5))
ConvXit
	movem.l	(a7)+,a3-a4
endproc


;
; Resources -----------------------------------------------------------------
;

	include	"pgmrcalc.res"
        dc.w    sysAppLaunchFlagNewStack|sysAppLaunchFlagNewGlobals|sysAppLaunchFlagUIApp|sysAppLaunchFlagSubCall
        dc.l    $1000                           ; stack size
        dc.l    $1000                           ; heap size

        res 'tver', kidrTVER
        dc.b    '1.0a1', 0

